@@ -721,6 +721,7 @@ class LdapManager:
                 if type(attrs) == dict:
                     results.append((dn, attrs))
 
+
             # Get cookie for the next request.
             paged_controls = self._get_pctrls(serverctrls)
             if not paged_controls:
@@ -780,7 +781,13 @@ class LdapManager:
             if v == self.pk:
                 dn_key = k
                 break
-        return "{}={},{}".format(dn_key, getattr(obj, self.pk), self.basedn)
+        pk_value = getattr(obj, self.pk)
+        if pk_value:
+            return "{}={},{}".format(dn_key, getattr(obj, self.pk), self.basedn)
+        else:
+            # If our pk_value is None or '', we're in the middle of creating a new record and haven't set
+            # it yet.
+            return None
 
     def disconnect(self):
         self.connection.unbind_s()
@@ -803,7 +810,6 @@ class LdapManager:
         ldap_object = ldap.initialize(config['url'])
         ldap_object.set_option(ldap.OPT_REFERRALS, 0)
         ldap_object.set_option(ldap.OPT_NETWORK_TIMEOUT, 15.0)
-        # if not settings.DEBUG:
         ldap.set_option(ldap.OPT_X_TLS, ldap.OPT_X_TLS_ALLOW)
         ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_NEVER)
         ldap_object.start_tls_s()
@@ -850,7 +856,6 @@ class LdapManager:
         for objectclass in self.extra_objectclasses:
             obj.objectclass.append(objectclass.encode())
         obj.objectclass.append(self.objectclass.encode())
-        # obj.objectclass = self.extra_objectclasses + [self.objectclass]
         modlist = Modlist(self).add(obj)
         self.connection.add_s(self.dn(obj), modlist)
 
@@ -879,13 +884,44 @@ class LdapManager:
 
         self.connection.delete_s(obj.dn)
 
+    @atomic(key='write')
+    def rename(self, old_dn, new_dn):
+        """
+        Update an object's dn, keeping it within the same basedn.
+        """
+        newrdn = new_dn.split(',')[0]
+        old_basedn = ','.join(old_dn.split(',')[1:])
+        new_basedn = ','.join(new_dn.split(',')[1:])
+        newsuperior = None
+        if old_basedn != new_basedn:
+            newsuperior = new_basedn
+        self.connection.rename_s(old_dn, newrdn, newsuperior)
+
     @atomic(key='write')
     def modify(self, obj, old=None):
-        # use a function that will use delete if value is None
+        # First check to see whether we updated our primary key.  If so, we need to rename
+        # the object in LDAP, and its obj._dn.  The old obj._dn should reference the old PK.
+        old_pk_value = obj.dn.split(',')[0].split('=')[1]
+        new_pk_value = getattr(obj, self.pk)
+        if new_pk_value != old_pk_value:
+            # We need to do a modrdn_s if we change pk, to cause the dn to be updated also
+            self.connection.modrdn_s(obj.dn, f'{self.pk}={new_pk_value}')
+            # And update our object's _dn to the new one
+            basedn = ','.join(obj.dn.split(',')[1:])
+            new_dn = f'{self.pk}={new_pk_value},{basedn}'
+            obj._dn = new_dn
+            # force reload old, if it was passed in so that we get the new pk value and dn
+            old = None
+
+        # Now update the non-PK attributes
         if not old:
             old = self.get(**{self.pk: getattr(obj, self.pk)})
         modlist = Modlist(self).update(obj, old)
-        self.connection.modify_s(obj.dn, modlist)
+        if modlist:
+            # Only issue the modify_s if we actually have changes
+            self.connection.modify_s(obj.dn, modlist)
+        else:
+            self.logger.debug(f'ldaporm.manager.modify.no-changes dn={obj.dn}')
 
     def only(self, *names):
         return F(manager=self).attributes(names)
